# -*- coding: utf-8 -*-
"""

Author: Brandon Hayes
Purpose: Control Manual Strobing Script (LED Light Source, camera exposure driven)
Notes:
    -- Cannot strobe faster than 30 Hz in this case due to long for loop times
    -- For reliability, can reduce for loop to only firing command so can do kHz firing frequencies
Things to Add:
    -- Automatic power supply calibration for analog. Always at 0 except when run
       (need voltmeter across voltage readout channel should be 0-5 V).
       calibrate using 0-30 V so within safe levels (or more if needed)
    -- Figure out digital power supply control for most safe operation
       (will be 0'ed, output disabled, except for when in use)
    -- Make gui control for each. Main GUI will open each section GUI that
       way don't need confusing tab codes
       (strobing, reliability, pre-warming, particles, power supply calibration)
    -- Use AD2 oscilloscope to measure capacitor voltage
    -- Use AD2 oscilloscope to measure shunt voltage from diff op amp, use 30 MHz,
       calculate the surface temperature each time, at end report surface temp plot vs time and save data
    -- Threaded
    -- Don't do real time showing of graphs animating, too slow; instead, use tkVideo to embed video
       after a run has completed when click "record" and when not "recording" use native pylon viewer
       https://stackoverflow.com/questions/7227162/way-to-play-video-files-in-tkinter
    -- Add automatic file naming convention if Folder Name is not the same as the base
       Use the parameters to name the file (important ones only) and then make a csv or text file
       that contains all the parameters used in the simulation and put it inside the image directory.
       When I make this the same exact way everytime, it allows me to do processing by pulling that
       file of information in each folder

Date Modified: 1-6-22
       
"""

# -------------------------------------------
# Import Modules
# -------------------------------------------

import tkinter as tk                    
from tkinter import ttk
import csv
import multiprocessing
#from multiprocessing import Process,Queue
import queue
import threading
from ctypes import *
from dwfconstants import * #need from AD2 folder in main directory
import math
import time
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import (FigureCanvasTkAgg, NavigationToolbar2Tk)
import sys
import numpy as np
import os
import statistics
from joblib import Parallel, delayed
from pypylon import pylon
from pypylon import genicam
import shutil
from PIL import Image, ImageTk
import PIL
#from numba import jit #to speed up non-numpy computations

#@jit #to speed up non-numpy computations (add before a function to define)

print( "\n" * 40) #acts as clear screen for idle

# -------------------------------------------
# Define Constants
# -------------------------------------------

data_length = 16384 #specify the data buffer length (corresponds to 3 config)
AD_frequency = 50e6 #specify the frequency (needed for how long can delay) divider = 1 --> 10 ns    
camera_pulse = 5e-6 #specify camera exposure time (triggers on rising edge)
oscilloscope_pulse = 5e-6 #specify oscilloscope pulse (triggers on rising edge)
starting_offset = 10e-6 #allow for the LED to turn on since the first picture was dark

p1 = 59.795799501839873 #slope (always leave the same)
p2 = 2.037965219092616 #y-intercept (will need to change as shifts over time)

total_duration_max = data_length/AD_frequency #re-define to the max number (can't put earlier cause not initiated)

# -------------------------------------------
# Make Queues
# -------------------------------------------

q = queue.Queue()
q_stop = queue.Queue()
q_stop_camera = queue.Queue()
q_stop.put(0) #start with false
q_stop_camera.put(0) #start with false
refresh_rate = 2
ID = -1

# -------------------------------------------
# Basler Camera Files
# -------------------------------------------

camera_config_directory = r"Basler Camera Settings"
nodeFile_UltraShort = "basler_binned.pfs"
nodeFile_Standard = "basler_binned_standard.pfs"
nodeFile_aim = "basler_binned_aim.pfs"

# -------------------------------------------
# Initialize Analog Discovery 2
# -------------------------------------------
   
if sys.platform.startswith("win"):
    dwf = cdll.dwf
elif sys.platform.startswith("darwin"):
    dwf = cdll.LoadLibrary("/Library/Frameworks/dwf.framework/dwf")
else:
    dwf = cdll.LoadLibrary("libdwf.so")

hdwf = c_int()
sts = c_byte()

version = create_string_buffer(16)
dwf.FDwfGetVersion(version)
print("DWF Version: "+str(version.value))

print("Opening first device")
dwf.FDwfDeviceConfigOpen(-1, 3, byref(hdwf)) #-1 means grab first device, 3 == 3rd configuration for max DIO buffer size

if hdwf.value == 0:
    print("failed to open device")
    szerr = create_string_buffer(512)
    dwf.FDwfGetLastErrorMsg(szerr)
    print(str(szerr.value))
    quit()
hzSys = c_double(100e6)
dwf.FDwfDigitalOutReset(hdwf)
print("Internal frequency is " + str(hzSys.value/1e6)+" MHz")

waveform_channel_1 = c_int(0) #channel 1 of waveform generator

# -------------------------------------------
# Define Functions
# -------------------------------------------

# ------ Refresh GUI Functions ------ #

def refresh_threaded(ID):
    
    #worker = multiprocessing.Process(target=edit_waveform,args=(ii,))    
    worker = threading.Thread(target=refresher,args=(ID,))
    worker.start()
    
def refresher(ID):
# only for strobing refresher, will need another refresher to update real time readings and switch
# to that refresher when not firing
    
    if q_stop.queue[0] == 1: #if true
        root.after_cancel(ID)
        q.queue.clear() #clear queue
        return
    
    light_step = float(light_step_entry.get())*1e-6
    total_duration = float(total_duration_entry.get())*1e-6
    total_light_offset = int(round(total_duration/light_step)) #how many iterations until reaches end
    
    if q.empty():
        ID = root.after(int(round(1e3*1/refresh_rate)), lambda: refresher(ID)) # queue empty, keep feeding in ID so can exit
    else:
        current_iteration,frequency_val = q.queue[-1] #get last item in queue since can't refresh as fast as items go into queues
        L_out_actual_frequency.configure(text="Actual Frequency: "+"{:.3f}".format(frequency_val)+" Hz")
        #make_waveform(current_iteration) #slows down the program too much
        #grab_image_for_gui(current_iteration)
        if current_iteration == 0: #last iteration only doing 1, exit refresher
            root.after_cancel(ID)
            q.queue.clear() #clear queue
            L_out_actual_frequency.config(text="Actual Frequency: "+ "{:.2f}".format(actual_execution_frequency)+"+/- " + "{:.2f}".format(std_actual_execution_frequency) +" Hz")
        else:
            ID = root.after(int(round(1e3*1/refresh_rate)), lambda: refresher(ID)) # every second...
        print("Refresh: " + str(current_iteration*light_step*1e6) + " out of " + str(light_step*total_light_offset*1e6) + " \u03bcs")

# ------ Camera Alignment Functions ------ #

def start_camera_threaded():
# want to multi-process so can be simultaneous
    
    camera_exposure = int(exposure_entry.get()) #already in us
    gain = int(gain_entry.get())
    digital_shift = int(digital_shift_entry.get())
    images_to_grab = int(1e8) #infinite loop
    
    #worker = multiprocessing.Process(target=start_camera,args=(camera_exposure,gain,digital_shift,images_to_grab,camera_config_directory,nodeFile_aim))
    worker = threading.Thread(target=start_camera,args=(camera_exposure,gain,digital_shift,images_to_grab,camera_config_directory,nodeFile_aim))
    worker.start()

def stop_camera_threaded():
    #worker = multiprocessing.Process(target=stop_camera())
    worker = threading.Thread(target=stop_camera)
    worker.start()

def start_camera(camera_exposure,gain,digital_shift,images_to_grab,camera_config_directory,nodeFile_aim):
# probably want this to be multiprocess so can run simaltaneously as gui
# and not have infinite loop take all resources and freeze the gui

    print('Alignment: Starting Basler Camera')
    q_stop_camera.queue.clear() #clear queue
    q_stop_camera.put(0) #re-set queue with false
            
    camera = pylon.InstantCamera(pylon.TlFactory.GetInstance().CreateFirstDevice())
    camera.Open()
    
    pylon.FeaturePersistence.Load(os.path.join(camera_config_directory,nodeFile_aim),camera.GetNodeMap(), True)

    camera.Gain.SetValue(gain) # was 10
    camera.DigitalShift.SetValue(digital_shift) #was 2

    imageWindow = pylon.PylonImageWindow()
    imageWindow.Create(1)
    camera.StartGrabbing(pylon.GrabStrategy_LatestImages) #start grabbing latest images

    for jj in range(images_to_grab): # nearly infinite loop
        if q_stop_camera.queue[0] == 1: #if true, exit multiprocessing loop if press stop
            q_stop_camera.queue.clear() #clear queue
            q_stop_camera.put(0) #re-set queue with false
            camera.Close()            
            imageWindow.Close()
            return
        grabResult = camera.RetrieveResult(5000, pylon.TimeoutHandling_ThrowException)
        if grabResult.GrabSucceeded():
            imageWindow.SetImage(grabResult)
            imageWindow.Show()
        else:
            print("Error: ",
                  grabResult.ErrorCode)  # grabResult.ErrorDescription does not work properly in python could throw UnicodeDecodeError
        grabResult.Release()

def stop_camera():
    print('Alignment: Stopping Basler Camera')
    q_stop_camera.queue.clear()
    q_stop_camera.put(1) #set to true
    return
    
# ------ Closing GUI Functions ------ #
    
def close_GUI_threaded():
    worker = threading.Thread(target=close_GUI)
    worker.start()

def close_GUI():
    set_power_supply_voltage(0)
    turn_off_AD_5V_supply()
    close_AD2()

    root.destroy() #end main loop of gui and stop all executables

# ------ Calibration Power Supply Functions ------ #
    
def powerSupplyOn_threaded():
    worker = threading.Thread(target=powerSupplyOn)
    worker.start()
    
def powerSupplyOff_threaded():
    worker = threading.Thread(target=powerSupplyOff)
    worker.start()

def powerSupplyOn():
    print("Calibration: Power Supply ON")
        
    global dwf,hdwf #to allow function access to global variable
    calibration_voltage = float(calibration_voltage_entry.get())
    set_power_supply_voltage(calibration_voltage)

def powerSupplyOff():
    print("Calibration: Power Supply OFF")
    
    global dwf,hdwf #to allow function access to global variable
    set_power_supply_voltage(0)
    
# ------ Main Start and Stop Functions ------ #

def start_threaded():
    q_stop.queue.clear()
    q_stop.put(0) #set to false, signal to allow loop to run

    worker = threading.Thread(target=start)
    worker.start()
    
def stop_threaded():
    worker = threading.Thread(target=stop)
    worker.start()
    
def start():

    refresh_threaded(ID) #keep refreshing in a separate thread
    run_manual_strobing_code_threaded()
        
def stop():
    print("STOP")
    q_stop.queue.clear() #clear queue
    q_stop.put(1) #set to true, put signal to terminate loops
    turn_off_AD_5V_supply()
    #close_basler_camera() #close the basler camera on stop (dont need for high speed)
    set_power_supply_voltage(0)

# ------ Make Waveform Plot Functions ------ #

def make_waveform(ii):

    voltage = float(voltage_entry.get())
    resistor_1_pulse = float(pulse_width_entry_1.get())*1e-6
    pre_pulse_width_1  = float(pre_pulse_width_entry_1.get())*1e-6
    pre_pulse_offset_1 = float(pre_pulse_offset_entry_1.get())*1e-6
    resistor_2_pulse = float(pulse_width_entry_2.get())*1e-6
    pre_pulse_width_2  = float(pre_pulse_width_entry_2.get())*1e-6
    pre_pulse_offset_2 = float(pre_pulse_offset_entry_2.get())*1e-6
    main_pulse_offset = float(main_pulse_offset_entry.get())*1e-6
    firing_frequency = float(frequency_entry.get())
    light_step = float(light_step_entry.get())*1e-6
    camera_delay = float(camera_time_delay_entry.get())*1e-6
    main_directory = main_directory_entry.get()
    folder_name = folder_name_entry.get()
    resistance = float(resistance_entry.get())
    total_duration = float(total_duration_entry.get())*1e-6
    light_pulse = float(light_pulse_entry.get())*1e-6
    camera_exposure = float(exposure_entry.get())*1e-6
    calibration_offset = float(calibration_offset_entry.get())
    calibration_voltage = float(calibration_voltage_entry.get())
    gain = float(gain_entry.get())
    digital_shift = float(digital_shift_entry.get())

    if voltage > 250: #set max limit at 250 so wont damage sensor and AD2
        voltage_entry.delete(0,tk.END)
        voltage_entry.insert(tk.END,str(250))
    
    if total_duration > (total_duration_max - camera_delay - camera_pulse - starting_offset): #can't have over the max length of the AD so correct
        total_duration_entry.delete(0,tk.END)
        total_duration_entry.insert(tk.END, str((total_duration_max - camera_delay - camera_pulse - starting_offset)*1e6)) #enter max number based on AD conditions
    
    folder_name = ("{:.2f}".format(voltage) + 'V, ' +
                  "{:.1f}".format(resistance) + 'R, ' +
                  "{:.1f}".format(resistor_1_pulse*1e6) + 'R1, ' +
                  "{:.1f}".format(resistor_2_pulse*1e6) + 'R2, ' +
                  "{:.1f}".format(light_pulse*1e6) + 'light, ' +
                  "{:.1f}".format(light_step*1e6) + 'step, ' +
                  "{:.1f}".format(camera_exposure*1e6) + 'exp, ' +
                  str(int(gain)) + 'G, ' +
                  str(int(digital_shift)) + 'DS')

    folder_name_entry.delete(0,tk.END)
    folder_name_entry.insert(tk.END, folder_name)

    time_vector_list = list(range(0,int(data_length)))
    time_vector_list = np.array(time_vector_list)
    time_vector = time_vector_list*(1/AD_frequency)
    
    channel_0 = np.zeros( (int(data_length)) ) #[time,state] Resistor 1, DIO 0
    channel_0[1+int(round((starting_offset+camera_delay)*AD_frequency)):1+int(round((starting_offset+camera_delay + pre_pulse_width_1)*AD_frequency))] = 1 #leave first state time at 0
    channel_0[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_delay)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_delay + resistor_1_pulse)*AD_frequency))] = 1 #leave first state time at 0              

    channel_3 = np.zeros( (int(data_length)) ) #[time,state] Resistor 2, DIO 4
    channel_3[1+int(round((main_pulse_offset+starting_offset+camera_delay)*AD_frequency)):1+int(round((main_pulse_offset+starting_offset+camera_delay + pre_pulse_width_2)*AD_frequency))] = 1 #leave first state time at 0
    channel_3[1+int(round((main_pulse_offset+pre_pulse_width_2+pre_pulse_offset_2+starting_offset+camera_delay)*AD_frequency)):1+int(round((main_pulse_offset+pre_pulse_width_2+pre_pulse_offset_2+starting_offset+camera_delay + resistor_2_pulse)*AD_frequency))] = 1 #leave first state time at 0    channel_3_rgbdata=(c_ubyte*((len(channel_3)+7)>>3))(0) # how many bytes we need to fit this many bits, (+7)/8    
    
    channel_4 = np.zeros( (int(data_length)) ) #[time,state] Oscilloscope, DIO 5
    channel_4[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_delay)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_delay + oscilloscope_pulse)*AD_frequency))] = 1 #leave first state time at 0

    channel_1 = np.zeros( (int(data_length)) ) #[time,state] Camera, DIO 1
    channel_1[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+light_step*ii)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_pulse + light_step*ii)*AD_frequency))] = 1 #leave first state time at 0
        
    channel_2 = np.zeros( (int(data_length)) ) #[time,state] Light, DIO 2
    if Camera_driven.get() == 1: #if camera driven checkbox
        channel_2[1:int(round((total_duration_max - 5e-6)*AD_frequency))] = 1 #leave first state time at 0 (LED normal light source)
    if LED_driven.get() == 1: #if LED driven checkbox
        channel_2[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_step*ii)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_pulse + light_step*ii)*AD_frequency))] = 1 #leave first state time at 0
    if High_speed_camera_driven.get() == 1: #if high speed camera checkbox
        duty_cycle = 0.01 # 1% duty cycle max with light source
        wait_time = light_pulse / duty_cycle # in seconds (time between pulses for a 1% duty cycle)
        number_of_pulses = math.floor(total_duration / wait_time) # number of light pulses in time period
        for kk in range(int(number_of_pulses)):
            channel_2[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_step*ii + wait_time*kk)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_pulse + light_step*ii + wait_time*kk)*AD_frequency))] = 1 #leave first state time at 0
            

    font_size = 14
    fig = Figure(figsize = (9,8.5),dpi=50)
    fig.clf()
    #plt.figure(1)
    #plt.clf()    
    #font = {'family' : 'normal',
    #        'weight' : 'bold',
    #        'size'   : 24}
    #plt.rc('font', **font)
    plt1 = fig.add_subplot(5,1,1)
    plt1.plot(time_vector*1e6,channel_0,'k',linewidth=2)
    #plt.set_xlabel('Time ($\mu$s)',fontweight='bold')
    plt1.set_ylabel('Channel 0\nResistor 1',fontweight='bold',rotation=90,fontsize=font_size)
    plt1.grid()
    plt1.tick_params(axis='both', which='major', labelsize=0)
    plt1.tick_params(axis='both', which='minor', labelsize=0)
    plt2 = fig.add_subplot(5,1,2)
    plt2.plot(time_vector*1e6,channel_1,'k',linewidth=2)
    #plt.set_xlabel('Time ($\mu$s)',fontweight='bold')
    plt2.set_ylabel('Channel 1\nCamera',fontweight='bold',rotation=90,fontsize=font_size)
    plt2.grid()
    plt2.tick_params(axis='both', which='major', labelsize=0)
    plt2.tick_params(axis='both', which='minor', labelsize=0)
    plt3 = fig.add_subplot(5,1,3)
    plt3.plot(time_vector*1e6,channel_2,'k',linewidth=2)
    #plt.set_xlabel('Time ($\mu$s)',fontweight='bold')
    plt3.set_ylabel('Channel 2\nLight',fontweight='bold',rotation=90,fontsize=font_size)
    plt3.grid()
    plt3.tick_params(axis='both', which='major', labelsize=0)
    plt3.tick_params(axis='both', which='minor', labelsize=0)    
    plt4 = fig.add_subplot(5,1,4)
    plt4.plot(time_vector*1e6,channel_3,'k',linewidth=2)
    #plt.set_xlabel('Time ($\mu$s)',fontweight='bold')
    plt4.set_ylabel('Channel 3\nResistor 2',fontweight='bold',rotation=90,fontsize=font_size)
    plt4.grid()
    plt4.tick_params(axis='both', which='major', labelsize=0)
    plt4.tick_params(axis='both', which='minor', labelsize=0)
    plt5 = fig.add_subplot(5,1,5)
    plt5.plot(time_vector*1e6,channel_4,'k',linewidth=2)
    plt5.set_xlabel('Time ($\mu$s)',fontweight='bold',fontsize=20)
    plt5.set_ylabel('Channel 4\nOscilloscope',fontweight='bold',rotation=90,fontsize=font_size)
    plt5.grid()
    plt5.tick_params(axis='x', which='major', labelsize=15)
    plt5.tick_params(axis='x', which='minor', labelsize=15)
    plt5.tick_params(axis='y', which='major', labelsize=0)
    plt5.tick_params(axis='y', which='minor', labelsize=0)
    canvas = FigureCanvasTkAgg(fig,master = root) #put in current selected tab
    canvas.draw()
    canvas.get_tk_widget().place(relx = .23,rely = .40)

# ------ Make Temperature Plot Functions ------ #
    
def make_temperature_plot():
    
    ii = 0
    time_vector_list = list(range(0,int(data_length)))
    time_vector_list = np.array(time_vector_list)
    time_vector = time_vector_list*(1/AD_frequency)

    voltage = float(voltage_entry.get())
    resistor_1_pulse = float(pulse_width_entry_1.get())*1e-6
    pre_pulse_width_1  = float(pre_pulse_width_entry_1.get())*1e-6
    pre_pulse_offset_1 = float(pre_pulse_offset_entry_1.get())*1e-6
    resistor_2_pulse = float(pulse_width_entry_2.get())*1e-6
    pre_pulse_width_2  = float(pre_pulse_width_entry_2.get())*1e-6
    pre_pulse_offset_2 = float(pre_pulse_offset_entry_2.get())*1e-6
    main_pulse_offset = float(main_pulse_offset_entry.get())*1e-6
    firing_frequency = float(frequency_entry.get())
    light_step = float(light_step_entry.get())*1e-6
    camera_delay = float(camera_time_delay_entry.get())*1e-6
    main_directory = main_directory_entry.get()
    folder_name = folder_name_entry.get()
    resistance = float(resistance_entry.get())
    total_duration = float(total_duration_entry.get())*1e-6
    light_pulse = float(light_pulse_entry.get())*1e-6
    camera_exposure = float(exposure_entry.get())*1e-6
    calibration_offset = float(calibration_offset_entry.get())
    calibration_voltage = float(calibration_voltage_entry.get())
    gain = float(gain_entry.get())
    digital_shift = float(digital_shift_entry.get())
    
    capacitor_voltage = 100
    sense_resistor_voltage = 0

    funcs = np.array([lambda x: capacitor_voltage*(1 - math.exp(-x/np.max(time_vector))) for i in range(int(data_length))])
    temperature = evaluate_function(funcs,time_vector)

    font_size = 14
    fig2 = Figure(figsize = (8,3.5),dpi=50)
    #plt.figure(1)
    #plt.clf()    
    #font = {'family' : 'normal',
    #        'weight' : 'bold',
    #        'size'   : 24}
    #plt.rc('font', **font)
    plt1 = fig2.add_subplot(1,1,1)
    plt1.plot(time_vector*1e6,temperature,'k',linewidth=2)
    plt1.set_xlabel('Time ($\mu$s)',fontweight='bold',fontsize=20)
    plt1.set_ylabel('Temperature ($^o$C)',fontweight='bold',rotation=90,fontsize=20)
    plt1.tick_params(axis='both', which='major', labelsize=15)
    plt1.tick_params(axis='both', which='minor', labelsize=15)
    plt1.grid()
    fig2.tight_layout()
    canvas2 = FigureCanvasTkAgg(fig2,master = root)  
    canvas2.draw()
    canvas2.get_tk_widget().place(relx = .56,rely = .40)

def evaluate_function(funcs, x):
    return np.array([func(val) for func, val in zip(funcs, x)])

# ------ Display Sample Bubble Image Functions ------ #

def grab_image_for_gui(ii):

    light_step = float(light_step_entry.get())*1e-6
       
    im = Image.open("example_image.png")
    im = im.resize((400,220),PIL.Image.ANTIALIAS)
    im = ImageTk.PhotoImage(im)
    im_label = tk.Label(root,image=im)
    im_label.photo = im
    im_label.place(relx = .56,rely = .68)
    
    time_label = tk.Label(root,text="t = " + "{:.2f}".format(ii*light_step*1e6) + " \u03bcs",font='Helvetica 14',fg='black',bg='white')
    time_label.place(relx = .57,rely = .69)

# ------ Strobing Code Functions ------ #

def run_manual_strobing_code_threaded():

    global dwf,hdwf,waveform_channel_1,hzSys,camera,imageWindow #so function can access global variables
    
    # For some reason, can't have R2 pre-pulse width be 0; seems to error on one of those combinations if try to have no R2 pulse
    
    voltage = float(voltage_entry.get())
    resistor_1_pulse = float(pulse_width_entry_1.get())*1e-6
    pre_pulse_width_1  = float(pre_pulse_width_entry_1.get())*1e-6
    pre_pulse_offset_1 = float(pre_pulse_offset_entry_1.get())*1e-6
    resistor_2_pulse = float(pulse_width_entry_2.get())*1e-6
    pre_pulse_width_2  = float(pre_pulse_width_entry_2.get())*1e-6
    pre_pulse_offset_2 = float(pre_pulse_offset_entry_2.get())*1e-6
    main_pulse_offset = float(main_pulse_offset_entry.get())*1e-6
    firing_frequency = float(frequency_entry.get())
    light_step = float(light_step_entry.get())*1e-6
    camera_delay = float(camera_time_delay_entry.get())*1e-6
    main_directory = main_directory_entry.get()
    folder_name = folder_name_entry.get()
    resistance = float(resistance_entry.get())
    total_duration = float(total_duration_entry.get())*1e-6
    light_pulse = float(light_pulse_entry.get())*1e-6
    camera_exposure = float(exposure_entry.get())*1e-6
    calibration_offset = float(calibration_offset_entry.get())
    calibration_voltage = float(calibration_voltage_entry.get())
    gain = float(gain_entry.get())
    digital_shift = float(digital_shift_entry.get())
    
    capacitor_voltage = 5
    magnetic_sensor = 0 #read from logic analyzer
    
    if capacitor_voltage < 10:
        L_out_capacitor.config(fg = 'green',text="Capacitor Voltage:"+" "+ str(capacitor_voltage) + " V")
    else:
        L_out_capacitor.config(fg = 'red',text="Capacitor Voltage:"+" "+ str(capacitor_voltage) + " V")

    if magnetic_sensor < 1:
        L_out_interlock.config(fg = 'red',text="Interlock: "+" "+"OPEN")
    else:
        L_out_interlock.config(fg = 'green',text="Interlock: "+" "+"CLOSED")
          
    time_vector_list = list(range(0,int(data_length)))
    time_vector_list = np.array(time_vector_list)
    time_vector = time_vector_list*(1/AD_frequency)
    
    total_light_offset = int(round(total_duration/light_step)) #how many iterations until reaches end
    strobing_vector = range(total_light_offset) #how many strobing events to do
    measured_time = np.zeros( total_light_offset )

    # -------------------------------------------
    # Make Stationary Waveforms (make and load in AD2 first)
    # -------------------------------------------
    
    channel_0 = np.zeros( (int(data_length)) ) #[time,state] Resistor 1, DIO 0
    channel_0[1+int(round((starting_offset+camera_delay)*AD_frequency)):1+int(round((starting_offset+camera_delay + pre_pulse_width_1)*AD_frequency))] = 1 #leave first state time at 0
    channel_0[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_delay)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_delay + resistor_1_pulse)*AD_frequency))] = 1 #leave first state time at 0
    channel_0_rgbdata=(c_ubyte*((len(channel_0)+7)>>3))(0) # how many bytes we need to fit this many bits, (+7)/8
    for i in range(len(channel_0)): # array to bits in byte array
        if channel_0[i] != 0:
            channel_0_rgbdata[i>>3] |= 1<<(i&7)              

    channel_5 = np.zeros( (int(data_length)) ) #[time,state] Resistor 2, DIO 5
    channel_5[1+int(round((main_pulse_offset+starting_offset+camera_delay)*AD_frequency)):1+int(round((main_pulse_offset+starting_offset+camera_delay + pre_pulse_width_2)*AD_frequency))] = 1 #leave first state time at 0
    channel_5[1+int(round((main_pulse_offset+pre_pulse_width_2+pre_pulse_offset_2+starting_offset+camera_delay)*AD_frequency)):1+int(round((main_pulse_offset+pre_pulse_width_2+pre_pulse_offset_2+starting_offset+camera_delay + resistor_2_pulse)*AD_frequency))] = 1 #leave first state time at 0    channel_3_rgbdata=(c_ubyte*((len(channel_3)+7)>>3))(0) # how many bytes we need to fit this many bits, (+7)/8    
    channel_5_rgbdata=(c_ubyte*((len(channel_5)+7)>>3))(0) # how many bytes we need to fit this many bits, (+7)/8
    for i in range(len(channel_5)): # array to bits in byte array
        if channel_5[i] != 0:
            channel_5_rgbdata[i>>3] |= 1<<(i&7)

    channel_4 = np.zeros( (int(data_length)) ) #[time,state] Oscilloscope, DIO 4
    channel_4[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_delay)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_delay + oscilloscope_pulse)*AD_frequency))] = 1 #leave first state time at 0
    channel_4_rgbdata=(c_ubyte*((len(channel_4)+7)>>3))(0) # how many bytes we need to fit this many bits, (+7)/8
    for i in range(len(channel_4)): # array to bits in byte array
        if channel_4[i] != 0:
            channel_4_rgbdata[i>>3] |= 1<<(i&7)
    
    pin = 0 #Resistor 1
    dwf.FDwfDigitalOutTypeSet(hdwf, c_int(pin), DwfDigitalOutTypeCustom)
    dwf.FDwfDigitalOutDividerSet(hdwf, c_int(pin), c_int(int(hzSys.value/AD_frequency))) # set sample rate
    dwf.FDwfDigitalOutDataSet(hdwf, c_int(pin), byref(channel_0_rgbdata), c_int(len(channel_0)))
    dwf.FDwfAnalogOutIdleSet(hdwf, c_int(pin), c_int(1)) # 1=DwfDigitalOutIdleLow, low when not running 
    dwf.FDwfDigitalOutEnableSet(hdwf, c_int(pin), c_int(1))

    pin = 5 #Resistor 2
    dwf.FDwfDigitalOutTypeSet(hdwf, c_int(pin), DwfDigitalOutTypeCustom)
    dwf.FDwfDigitalOutDividerSet(hdwf, c_int(pin), c_int(int(hzSys.value/AD_frequency))) # set sample rate
    dwf.FDwfDigitalOutDataSet(hdwf, c_int(pin), byref(channel_5_rgbdata), c_int(len(channel_5)))
    dwf.FDwfAnalogOutIdleSet(hdwf, c_int(pin), c_int(1)) # 1=DwfDigitalOutIdleLow, low when not running 
    dwf.FDwfDigitalOutEnableSet(hdwf, c_int(pin), c_int(1))

    pin = 4 #Oscilloscope
    dwf.FDwfDigitalOutTypeSet(hdwf, c_int(pin), DwfDigitalOutTypeCustom)
    dwf.FDwfDigitalOutDividerSet(hdwf, c_int(pin), c_int(int(hzSys.value/AD_frequency))) # set sample rate
    dwf.FDwfDigitalOutDataSet(hdwf, c_int(pin), byref(channel_4_rgbdata), c_int(len(channel_4)))
    dwf.FDwfAnalogOutIdleSet(hdwf, c_int(pin), c_int(1)) # 1=DwfDigitalOutIdleLow, low when not running 
    dwf.FDwfDigitalOutEnableSet(hdwf, c_int(pin), c_int(1))
    
    
    # -------------------------------------------
    # Make Moving Waveforms
    # -------------------------------------------    

    # Turn on Camera, AD 5V Supply, Power Supply
    
    #start_basler_camera()
    turn_on_AD_5V_supply()
    set_power_supply_voltage(voltage)
    
    startt = time.perf_counter_ns() #re-initialize at each loop so corrects drift
    for ii in range(1): #only fire once for high speed
        if q_stop.queue[0] == 1: #if true, to allow break out of loop
            q.queue.clear()
            q_stop.queue.clear()
            q_stop.put(0) #set to false, reset queue to false to allow next cycle
            return
        #print(str(ii*light_step*1e6) + " out of " + str(light_step*total_light_offset*1e6) + "\u03bcs")
        
        channel_1 = np.zeros( (int(data_length)) ) #[time,state] Camera, DIO 1
        channel_1[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+light_step*ii)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset+camera_pulse + light_step*ii)*AD_frequency))] = 1 #leave first state time at 0
        channel_1_rgbdata=(c_ubyte*((len(channel_1)+7)>>3))(0) # how many bytes we need to fit this many bits, (+7)/8
        for i in range(len(channel_1)): # array to bits in byte array
            if channel_1[i] != 0:
                channel_1_rgbdata[i>>3] |= 1<<(i&7)
            
        channel_2 = np.zeros( (int(data_length)) ) #[time,state] Light, DIO 2
        channel_7 = np.zeros( (int(data_length)) ) #[time,state] High Speed Camera, DIO 7
        if Camera_driven.get() == 1: #if camera driven checkbox
            channel_2[1:int(round((total_duration - 5e-6)*AD_frequency))] = 1 #leave first state time at 0 (LED normal light source)
        if LED_driven.get() == 1: #if LED driven checkbox
            channel_2[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_step*ii)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_pulse + light_step*ii)*AD_frequency))] = 1 #leave first state time at 0
        if High_speed_camera_driven.get() == 1: #if high speed camera checkbox
            duty_cycle = 0.01 # 1% duty cycle max with light source
            wait_time = light_pulse / duty_cycle # in seconds (time between pulses for a 1% duty cycle)
            camera_on_signal = 4e-6 #needs to be at least 3 us long
            number_of_pulses = math.floor(total_duration / wait_time) # number of light pulses in time period
            fudge_factor = 0e-6
            for kk in range(int(number_of_pulses)):
                channel_2[1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_step*ii + wait_time*kk)*AD_frequency)):1+int(round((pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_pulse + light_step*ii + wait_time*kk)*AD_frequency))] = 1 #leave first state time at 0
                channel_7[1+int(round((fudge_factor + pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_step*ii + wait_time*kk)*AD_frequency)):1+int(round((fudge_factor + pre_pulse_width_1+pre_pulse_offset_1+starting_offset + camera_delay + light_pulse + light_step*ii + wait_time*kk + camera_on_signal)*AD_frequency))] = 1 #leave first state time at 0

        channel_2_rgbdata=(c_ubyte*((len(channel_2)+7)>>3))(0) # how many bytes we need to fit this many bits, (+7)/8
        for i in range(len(channel_2)): # array to bits in byte array
            if channel_2[i] != 0:
                channel_2_rgbdata[i>>3] |= 1<<(i&7)

        channel_7_rgbdata=(c_ubyte*((len(channel_7)+7)>>3))(0) # how many bytes we need to fit this many bits, (+7)/8
        for i in range(len(channel_7)): # array to bits in byte array
            if channel_7[i] != 0:
                channel_7_rgbdata[i>>3] |= 1<<(i&7)

        #Generate Patterns
        pin = 1 #Camera, DIO 1
        dwf.FDwfDigitalOutTypeSet(hdwf, c_int(pin), DwfDigitalOutTypeCustom)
        dwf.FDwfDigitalOutDividerSet(hdwf, c_int(pin), c_int(int(hzSys.value/AD_frequency))) # set sample rate
        dwf.FDwfDigitalOutDataSet(hdwf, c_int(pin), byref(channel_1_rgbdata), c_int(len(channel_1)))
        dwf.FDwfAnalogOutIdleSet(hdwf, c_int(pin), c_int(1)) # 1=DwfDigitalOutIdleLow, low when not running 
        dwf.FDwfDigitalOutEnableSet(hdwf, c_int(pin), c_int(1))
        
        pin = 2 #Light, DIO 2
        dwf.FDwfDigitalOutTypeSet(hdwf, c_int(pin), DwfDigitalOutTypeCustom)
        dwf.FDwfDigitalOutDividerSet(hdwf, c_int(pin), c_int(int(hzSys.value/AD_frequency))) # set sample rate
        dwf.FDwfDigitalOutDataSet(hdwf, c_int(pin), byref(channel_2_rgbdata), c_int(len(channel_2)))
        dwf.FDwfAnalogOutIdleSet(hdwf, c_int(pin), c_int(1)) # 1=DwfDigitalOutIdleLow, low when not running 
        dwf.FDwfDigitalOutEnableSet(hdwf, c_int(pin), c_int(1))

        pin = 7 #High Speed Camera, DIO 7 (same as the light signal)
        dwf.FDwfDigitalOutTypeSet(hdwf, c_int(pin), DwfDigitalOutTypeCustom)
        dwf.FDwfDigitalOutDividerSet(hdwf, c_int(pin), c_int(int(hzSys.value/AD_frequency))) # set sample rate
        dwf.FDwfDigitalOutDataSet(hdwf, c_int(pin), byref(channel_7_rgbdata), c_int(len(channel_7)))
        dwf.FDwfAnalogOutIdleSet(hdwf, c_int(pin), c_int(1)) # 1=DwfDigitalOutIdleLow, low when not running 
        dwf.FDwfDigitalOutEnableSet(hdwf, c_int(pin), c_int(1))
        
        stopt = time.perf_counter_ns()
        diff = (stopt - startt)/1e9
        while diff <= 1/firing_frequency:
            stopt = time.perf_counter_ns()
            diff = (stopt - startt)/1e9
        
        dwf.FDwfDigitalOutRepeatSet(hdwf, c_int(1)) # once
        dwf.FDwfDigitalOutRunSet(hdwf, c_double(total_duration_max)) # run time  (don't repeat after finish)       
        dwf.FDwfDigitalOutConfigure(hdwf, c_int(1)) #fire
        
        #if should_record.get() == 0: #if false, only read out in real time if not need to record
        #    read_out_non_recording_basler_camera()
        
        startt = time.perf_counter_ns() #re-initialize at each loop so corrects drift
        q.put([ii,1/diff]) #put in queue        
        measured_time[ii] = diff
        print('Done with Single Pulse')

    global actual_execution_frequency,std_actual_execution_frequency      
    #actual_execution_time = statistics.mean(measured_time)
    #actual_execution_frequency = 1/actual_execution_time
    #std_actual_execution_frequency = statistics.stdev(1/measured_time)
    actual_execution_frequency = 0
    std_actual_execution_frequency = 0
    
    # Turn Off System When Done
    #close_basler_camera()
    set_power_supply_voltage(0) #zero power supply at end of test
    turn_off_AD_5V_supply()

# ------ Set AD2 and Main Power Supply Functions ------ #

def set_power_supply_voltage(desired_voltage):
    print("Set Power Supply Voltage at " + str(desired_voltage) + " V")
    global dwf,hdwf,waveform_channel_1 #to allow function access to global variable
    
    calibration_offset = float(calibration_offset_entry.get())
    
    AD_voltage = (desired_voltage - p2 + calibration_offset)/p1 #calculate the needed AD voltage (from past calibration)
    dwf.FDwfAnalogOutNodeEnableSet(hdwf, waveform_channel_1, AnalogOutNodeCarrier, c_bool(True))
    dwf.FDwfAnalogOutIdleSet(hdwf, waveform_channel_1, DwfAnalogOutIdleOffset)
    dwf.FDwfAnalogOutNodeFunctionSet(hdwf, waveform_channel_1, AnalogOutNodeCarrier, funcDC)
    dwf.FDwfAnalogOutNodeOffsetSet(hdwf, waveform_channel_1, AnalogOutNodeCarrier, c_double(AD_voltage))
    dwf.FDwfAnalogOutConfigure(hdwf, waveform_channel_1, c_bool(True))
    time.sleep(3) #give 3 seconds to respond

def turn_on_AD_5V_supply():
    print('Turn On AD2 5V Supply')
    global dwf,hdwf #to allow function access to global variable
    
    # enable positive supply
    dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(0), c_int(0), c_double(True)) 
    # set voltage to 5 V
    dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(0), c_int(1), c_double(5.0)) 
    # enable negative supply
    #dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(1), c_int(0), c_double(True)) 
    # set voltage to 0 V
    #dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(1), c_int(1), c_double(0)) 
    # master enable
    dwf.FDwfAnalogIOEnableSet(hdwf, c_int(True))

def turn_off_AD_5V_supply():
    print('Turn Off AD2 5V Supply')
    global dwf,hdwf #to allow function access to global variable
    
    # enable positive supply
    dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(0), c_int(0), c_double(False)) 
    # set voltage to 5 V
    dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(0), c_int(1), c_double(0.0)) 
    # enable negative supply
    #dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(1), c_int(0), c_double(False)) 
    # set voltage to 0 V
    #dwf.FDwfAnalogIOChannelNodeSet(hdwf, c_int(1), c_int(1), c_double(0)) 
    # master enable
    dwf.FDwfAnalogIOEnableSet(hdwf, c_int(False))

def close_AD2():
    print('Close AD2')
    global dwf #to allow function access to global variable
    
    dwf.FDwfDeviceCloseAll()
    
# ------ Basler Camera Functions ------ #

def start_basler_camera():
    print('Starting Basler Camera')
    global camera,imageWindow #to allow function access to global variable

    voltage = float(voltage_entry.get())
    resistor_1_pulse = float(pulse_width_entry_1.get())*1e-6
    pre_pulse_width_1  = float(pre_pulse_width_entry_1.get())*1e-6
    pre_pulse_offset_1 = float(pre_pulse_offset_entry_1.get())*1e-6
    resistor_2_pulse = float(pulse_width_entry_2.get())*1e-6
    pre_pulse_width_2  = float(pre_pulse_width_entry_2.get())*1e-6
    pre_pulse_offset_2 = float(pre_pulse_offset_entry_2.get())*1e-6
    main_pulse_offset = float(main_pulse_offset_entry.get())*1e-6
    firing_frequency = float(frequency_entry.get())
    light_step = float(light_step_entry.get())*1e-6
    camera_delay = float(camera_time_delay_entry.get())*1e-6
    main_directory = main_directory_entry.get()
    folder_name = folder_name_entry.get()
    resistance = float(resistance_entry.get())
    total_duration = float(total_duration_entry.get())*1e-6
    light_pulse = float(light_pulse_entry.get())*1e-6
    camera_exposure = int(exposure_entry.get())
    calibration_offset = float(calibration_offset_entry.get())
    calibration_voltage = float(calibration_voltage_entry.get())
    gain = int(gain_entry.get())
    digital_shift = int(digital_shift_entry.get())
    
    total_light_offset = int(round(total_duration/light_step)) #how many iterations until reaches end
    images_to_grab = int(round(total_light_offset))

    camera = pylon.InstantCamera(pylon.TlFactory.GetInstance().CreateFirstDevice())
    camera.Open()
    
    if camera_exposure <= 14: #need to shift to standard mode (in us)
        print("Using Binning 2, UltraShort Exposure Mode (1-14 us)")
        pylon.FeaturePersistence.Load(os.path.join(camera_config_directory,nodeFile_UltraShort), camera.GetNodeMap(), True)
    else:
        print("Using Binning 2, Standard Exposure Mode (>14 us)")     
        pylon.FeaturePersistence.Load(os.path.join(camera_config_directory,nodeFile_Standard),camera.GetNodeMap(), True)

    camera.MaxNumBuffer = images_to_grab
    camera.ExposureTime.SetValue(camera_exposure) #in us (0-14 in ultrashort)
    camera.Gain.SetValue(gain) # was 10
    camera.DigitalShift.SetValue(digital_shift) #was 2

    if should_record.get() == 1: # if true, only save if need to record
        camera.StartGrabbingMax(images_to_grab, pylon.GrabStrategy_OneByOne)
    else:
        camera.StartGrabbingMax(images_to_grab, pylon.GrabStrategy_LatestImages)
        imageWindow = pylon.PylonImageWindow()
        imageWindow.Create(1)    

def read_out_basler_camera():    
    print('Reading Out Basler Camera')
    global camera
    
    voltage = float(voltage_entry.get())
    resistor_1_pulse = float(pulse_width_entry_1.get())*1e-6
    pre_pulse_width_1  = float(pre_pulse_width_entry_1.get())*1e-6
    pre_pulse_offset_1 = float(pre_pulse_offset_entry_1.get())*1e-6
    resistor_2_pulse = float(pulse_width_entry_2.get())*1e-6
    pre_pulse_width_2  = float(pre_pulse_width_entry_2.get())*1e-6
    pre_pulse_offset_2 = float(pre_pulse_offset_entry_2.get())*1e-6
    main_pulse_offset = float(main_pulse_offset_entry.get())*1e-6
    firing_frequency = float(frequency_entry.get())
    light_step = float(light_step_entry.get())*1e-6
    camera_delay = float(camera_time_delay_entry.get())*1e-6
    main_directory = main_directory_entry.get()
    folder_name = folder_name_entry.get()
    resistance = float(resistance_entry.get())
    total_duration = float(total_duration_entry.get())*1e-6
    light_pulse = float(light_pulse_entry.get())*1e-6
    camera_exposure = int(exposure_entry.get()) #already in us
    calibration_offset = float(calibration_offset_entry.get())
    calibration_voltage = float(calibration_voltage_entry.get())
    gain = int(gain_entry.get())
    digital_shift = int(digital_shift_entry.get())

    total_light_offset = int(round(total_duration/light_step)) #how many iterations until reaches end
    images_to_grab = int(round(total_light_offset))

    if os.path.exists(os.path.join(main_directory,folder_name)):
        shutil.rmtree(os.path.join(main_directory,folder_name))
    os.makedirs(os.path.join(main_directory,folder_name))

    for i in range(images_to_grab):
        filename = os.path.join(main_directory,folder_name,"image_" + give_image_number(i,images_to_grab) + '.tiff')
        grabResult = camera.RetrieveResult(500, pylon.TimeoutHandling_Return)
        if grabResult.IsValid():
            img = grabResult.Array
            img = Image.fromarray(img)
            img.save(filename)
        else:
            return
        grabResult.Release()

def read_out_non_recording_basler_camera(): #read out frame of basler camera to imageWindow
    global camera,imageWindow
    
    grabResult = camera.RetrieveResult(500, pylon.TimeoutHandling_ThrowException)
    if grabResult.IsValid():
        imageWindow.SetImage(grabResult)
        imageWindow.Show()
        grabResult.Release()
    else:
        pass
        
def close_basler_camera():
    print("Closing Basler Camera (and read out if needed)")
    
    global imageWindow,camera #to allow function access to global variable

    if should_record.get() == 1: #if true, record images
        read_out_basler_camera()
        camera.Close()
    else: #just close the image window
        camera.Close()
        imageWindow.Close()

def give_image_number(im_num,max_num):
    num_str = str(im_num)
    str_length = len(num_str)
    num_str_max = str(max_num)
    str_length_max = len(num_str_max)
    padding_zeros_length = str_length_max - str_length
    new_num = (str(0) * padding_zeros_length) + str(im_num)
    return new_num

# -------------------------------------------
# Construct GUI
# -------------------------------------------

# Initiate Tkinter window
root = tk.Tk()        
root.geometry('1400x700')
root.title("Bubble Control GUI: High Speed Camera")

# -------------------------------------------
# Define Tab 1: Manual Strobing
# -------------------------------------------

# ------ Pulse Parameter Section ------ #

L0=tk.Label(root, text="Pulse Parameters", bg='lightblue',font='Helvetica 16 bold')
L0.grid(row=0, column=0, padx=0, pady=5, columnspan=2) #span 2 so it does not affect component button
 
L1=tk.Label(root,text="Voltage:",font='Helvetica 14')
L1.grid(row=1, column=0, padx=0,pady=5, columnspan =1)
voltage_entry= tk.Entry(root,font=('Century 12'),width=5)
voltage_entry.grid(row=1,column=1)
voltage_entry.insert(tk.END, '0')
L1_1=tk.Label(root,text="V",font='Helvetica 14')
L1_1.grid(row=1, column=2, padx=0,pady=5, columnspan =1)    

# Resistor 1:    
L2=tk.Label(root,text="R1 - Pulse Width:",font='Helvetica 14')
L2.grid(row=2, column=0, padx=0,pady=5, columnspan =1)
pulse_width_entry_1= tk.Entry(root,font=('Century 12'),width=5)
pulse_width_entry_1.grid(row=2,column=1)
pulse_width_entry_1.insert(tk.END, '5')    
L2_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L2_1.grid(row=2, column=2, padx=0,pady=5, columnspan =1)

L3=tk.Label(root,text="R1 - Pre-pulse Width:",font='Helvetica 14')
L3.grid(row=3, column=0, padx=0,pady=5, columnspan =1)
pre_pulse_width_entry_1= tk.Entry(root,font=('Century 12'),width=5)
pre_pulse_width_entry_1.grid(row=3,column=1)
pre_pulse_width_entry_1.insert(tk.END, '0')        
L3_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L3_1.grid(row=3, column=2, padx=0,pady=5, columnspan =1)

L4=tk.Label(root,text="R1 - Pre-pulse Offset:",font='Helvetica 14')
L4.grid(row=4, column=0, padx=0,pady=5, columnspan =1)
pre_pulse_offset_entry_1= tk.Entry(root,font=('Century 12'),width=5)
pre_pulse_offset_entry_1.grid(row=4,column=1)
pre_pulse_offset_entry_1.insert(tk.END, '0')            
L4_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L4_1.grid(row=4, column=2, padx=0,pady=5, columnspan =1)

# Resistor 2:
L2=tk.Label(root,text="R2 - Pulse Width:",font='Helvetica 14')
L2.grid(row=5, column=0, padx=0,pady=5, columnspan =1)
pulse_width_entry_2= tk.Entry(root,font=('Century 12'),width=5)
pulse_width_entry_2.grid(row=5,column=1)
pulse_width_entry_2.insert(tk.END, '0')    
L2_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L2_1.grid(row=5, column=2, padx=0,pady=5, columnspan =1)

L3=tk.Label(root,text="R2 - Pre-pulse Width:",font='Helvetica 14')
L3.grid(row=6, column=0, padx=0,pady=5, columnspan =1)
pre_pulse_width_entry_2= tk.Entry(root,font=('Century 12'),width=5)
pre_pulse_width_entry_2.grid(row=6,column=1)
pre_pulse_width_entry_2.insert(tk.END, '0')        
L3_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L3_1.grid(row=6, column=2, padx=0,pady=5, columnspan =1)

L4=tk.Label(root,text="R2 - Pre-pulse Offset:",font='Helvetica 14')
L4.grid(row=7, column=0, padx=0,pady=5, columnspan =1)
pre_pulse_offset_entry_2= tk.Entry(root,font=('Century 12'),width=5)
pre_pulse_offset_entry_2.grid(row=7,column=1)
pre_pulse_offset_entry_2.insert(tk.END, '0')            
L4_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L4_1.grid(row=7, column=2, padx=0,pady=5, columnspan =1)

L4=tk.Label(root,text="Main Pulse Offset:",font='Helvetica 14')
L4.grid(row=8, column=0, padx=0,pady=5, columnspan =1)
main_pulse_offset_entry= tk.Entry(root,font=('Century 12'),width=5)
main_pulse_offset_entry.grid(row=8,column=1)
main_pulse_offset_entry.insert(tk.END, '0')            
L4_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L4_1.grid(row=8, column=2, padx=0,pady=5, columnspan =1)

# ------ Firing Parameter Section ------ #

Ls1=tk.Label(root, text="    ") #add spacing
Ls1.grid(row=0, column=3, padx=0, pady=5, columnspan=1) #span 2 so it does not affect component butto

L5=tk.Label(root, text="Firing Parameters", bg='lightblue',font='Helvetica 16 bold')
L5.grid(row=0, column=4, padx=0, pady=5, columnspan=2) #span 2 so it does not affect component butto

L6=tk.Label(root,text="Frequency:",font='Helvetica 14')
L6.grid(row=1, column=4, padx=0,pady=5, columnspan =1)
frequency_entry= tk.Entry(root,font=('Century 12'),width=5)
frequency_entry.grid(row=1,column=5)
frequency_entry.insert(tk.END, '2')
L6_1=tk.Label(root,text="Hz",font='Helvetica 14')
L6_1.grid(row=1, column=6, padx=0,pady=5, columnspan =1)

L6=tk.Label(root,text="Total Duration:",font='Helvetica 14')
L6.grid(row=2, column=4, padx=0,pady=5, columnspan =1)
total_duration_entry= tk.Entry(root,font=('Century 12'),width=5)
total_duration_entry.grid(row=2,column=5)
total_duration_entry.insert(tk.END, str(total_duration_max*1e6)) #enter max number based on AD conditions

L6_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L6_1.grid(row=2, column=6, padx=0,pady=5, columnspan =1)

# ------ Camera Parameter Section ------ #

Ls2=tk.Label(root, text="    ") #add spacing
Ls2.grid(row=0, column=7, padx=0, pady=5, columnspan=1) #span 2 so it does not affect component butto

L7=tk.Label(root, text="Camera Parameters", bg='lightblue',font='Helvetica 16 bold')
L7.grid(row=0, column=8, padx=0, pady=5, columnspan=2) #span 2 so it does not affect component butto

L8=tk.Label(root,text="Time Delay:",font='Helvetica 14')
L8.grid(row=1, column=8, padx=0,pady=5, columnspan =1)
camera_time_delay_entry= tk.Entry(root,font=('Century 12'),width=5)
camera_time_delay_entry.grid(row=1,column=9)
camera_time_delay_entry.insert(tk.END, '36')
L8_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L8_1.grid(row=1, column=10, padx=0,pady=5, columnspan =1)

L8=tk.Label(root,text="Light Pulse:",font='Helvetica 14')
L8.grid(row=2, column=8, padx=0,pady=5, columnspan =1)
light_pulse_entry= tk.Entry(root,font=('Century 12'),width=5)
light_pulse_entry.grid(row=2,column=9)
light_pulse_entry.insert(tk.END, '1')
L8_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L8_1.grid(row=2, column=10, padx=0,pady=5, columnspan =1)

L8=tk.Label(root,text="Light Step:",font='Helvetica 14')
L8.grid(row=3, column=8, padx=0,pady=5, columnspan =1)
light_step_entry= tk.Entry(root,font=('Century 12'),width=5)
light_step_entry.grid(row=3,column=9)
light_step_entry.insert(tk.END, '1')
L8_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L8_1.grid(row=3, column=10, padx=0,pady=5, columnspan =1)

L8=tk.Label(root,text="Exposure:",font='Helvetica 14')
L8.grid(row=4, column=8, padx=0,pady=5, columnspan =1)
exposure_entry= tk.Entry(root,font=('Century 12'),width=5)
exposure_entry.grid(row=4,column=9)
exposure_entry.insert(tk.END, '1')
L8_1=tk.Label(root,text="\u03bcs",font='Helvetica 14')
L8_1.grid(row=4, column=10, padx=0,pady=5, columnspan =1)

L8=tk.Label(root,text="Gain:",font='Helvetica 14')
L8.grid(row=5, column=8, padx=0,pady=5, columnspan =1)
gain_entry = tk.Entry(root,font=('Century 12'),width=5)
gain_entry.grid(row=5,column=9)
gain_entry.insert(tk.END, '0')

L8=tk.Label(root,text="Digital Shift:",font='Helvetica 14')
L8.grid(row=6, column=8, padx=0,pady=5, columnspan =1)
digital_shift_entry = tk.Entry(root,font=('Century 12'),width=5)
digital_shift_entry.grid(row=6,column=9)
digital_shift_entry.insert(tk.END, '0')

# ------ File Saving Section ------ #

Ls3=tk.Label(root, text="    ") #add spacing
Ls3.grid(row=0, column=11, padx=0, pady=5, columnspan=1) #span 2 so it does not affect component butto

L9=tk.Label(root, text="Save Location", bg='lightblue',font='Helvetica 16 bold')
L9.grid(row=0, column=12, padx=0, pady=5, columnspan=1) #span 2 so it does not affect component butto

L10=tk.Label(root,text="Main Directory:",font='Helvetica 14')
L10.grid(row=1, column=12, padx=0,pady=5, columnspan =1)
main_directory_entry= tk.Entry(root,font=('Century 12'),width=20)
main_directory_entry.grid(row=1,column=13)
main_directory_entry.insert(tk.END, r"C:\Users\Brandon\Documents\Bubble GUI\Basler Camera Settings")

L11=tk.Label(root,text="Folder Name:",font='Helvetica 14')
L11.grid(row=2, column=12, padx=0,pady=5, columnspan =1)
folder_name_entry= tk.Entry(root,font=('Century 12'),width=20)
folder_name_entry.grid(row=2,column=13)
folder_name_entry.insert(tk.END, 'Test Folder')

L12=tk.Label(root,text="Resistance:",font='Helvetica 14')
L12.grid(row=3, column=12, padx=0,pady=5, columnspan =1)
resistance_entry= tk.Entry(root,font=('Century 12'),width=20)
resistance_entry.grid(row=3,column=13)
resistance_entry.insert(tk.END, '0')    
L12_1=tk.Label(root,text="\u03A9",font='Helvetica 14')
L12_1.grid(row=3, column=14, padx=0,pady=5, columnspan =1)

# ------ Power Supply Calibration Section ------ #

Ls3=tk.Label(root, text="    ") #add spacing
Ls3.grid(row=0, column=15, padx=0, pady=5, columnspan=1) #span 2 so it does not affect component butto

L9=tk.Label(root, text="Calibration", bg='lightblue',font='Helvetica 16 bold')
L9.grid(row=0, column=16, padx=0, pady=5, columnspan=2) #span 2 so it does not affect component butto

L10=tk.Label(root,text="Offset:",font='Helvetica 14')
L10.grid(row=1, column=16, padx=0,pady=5, columnspan =1)
calibration_offset_entry= tk.Entry(root,font=('Century 12'),width=5)
calibration_offset_entry.grid(row=1,column=18)
calibration_offset_entry.insert(tk.END, str(-1.3))
L12_1=tk.Label(root,text="V",font='Helvetica 14')
L12_1.grid(row=1, column=19, padx=0,pady=5, columnspan =1)

L10=tk.Label(root,text="Voltage Set:",font='Helvetica 14')
L10.grid(row=2, column=16, padx=0,pady=5, columnspan =1)
calibration_voltage_entry= tk.Entry(root,font=('Century 12'),width=5)
calibration_voltage_entry.grid(row=2,column=18)
calibration_voltage_entry.insert(tk.END, str(0))
L12_1=tk.Label(root,text="V",font='Helvetica 14')
L12_1.grid(row=2, column=19, padx=0,pady=5, columnspan =1)

powerSupplyOn_button = tk.Button(root, text = 'On',font='Helvetica 18', bg='green', activebackground='#00FF00',command=lambda: powerSupplyOn_threaded())
powerSupplyOn_button.place(relx = .85,rely = .2)
powerSupplyOff_button = tk.Button(root, text = 'Off',font='Helvetica 18', bg='red', activebackground='#ffcccb',command=lambda: powerSupplyOff_threaded())
powerSupplyOff_button.place(relx = 0.9,rely = .2)

# ------ Read Output Section ------ #

Lout0=tk.Label(root, text="Realtime Reading", bg='lightblue',font='Helvetica 16 bold')
Lout0.place(relx = 0.005,rely = .53)    

L_out_capacitor=tk.Label(root,text="Capacitor Voltage:"+" "+"0 V",font='Helvetica 14')
L_out_capacitor.place(relx = 0.005,rely = .58)    #config() update so color is red vs green if safe (safe is < 10 V)

L_out_temperature=tk.Label(root,text="Max Temperature:"+" "+"0 " + "\u00b0" + "C",font='Helvetica 14')
L_out_temperature.place(relx = 0.005,rely = .63)

L_out_interlock=tk.Label(root,text="Interlock: "+" "+"OPEN",font='Helvetica 14')
L_out_interlock.place(relx = 0.005,rely = .68)    #config() update so color is red vs green if safe (safe is < 1 V)

L_out_actual_frequency=tk.Label(root,text="Actual Frequency: "+str(0)+" Hz",font='Helvetica 14')
L_out_actual_frequency.place(relx = 0.005,rely = .73)    #config() update so color is red vs green if safe (safe is < 1 V)

# ------ Start, Stop, Set Buttons ------ #

startbutton = tk.Button(root, text = 'Start',font='Helvetica 18', bg='green', activebackground='#00FF00',command=lambda: start_threaded())
startbutton.place(relx = 0,rely = .9)

stopbutton = tk.Button(root, text = 'Stop',font='Helvetica 18', bg='red', activebackground='#ffcccb',command=lambda: stop_threaded())
stopbutton.place(relx = 0.1,rely = .9)

closebutton = tk.Button(root, text = 'Close',font='Helvetica 18', bg='yellow', activebackground='#F0F000',command=lambda: close_GUI_threaded())
closebutton.place(relx = 0.9,rely = .9)

setbutton = tk.Button(root, text = 'Set',font='Helvetica 12',command=lambda: make_waveform(0)) #zero case for ii=0
setbutton.grid(row=8,column=3)

Lout0=tk.Label(root, text="Camera Alignment:", bg='lightblue',font='Helvetica 16 bold')
Lout0.place(relx = 0.85,rely = .40)
startCamerabutton = tk.Button(root, text = 'Start Camera',font='Helvetica 14', bg='green', activebackground='#00FF00',command=lambda: start_camera_threaded())
startCamerabutton.place(relx = 0.85,rely = .45)
stopCamerabutton = tk.Button(root, text = 'Stop Camera',font='Helvetica 14', bg='red', activebackground='#ffcccb',command=lambda: stop_camera_threaded())
stopCamerabutton.place(relx = 0.85,rely = .53)

# ------ Checkbox Buttons ------ #

Camera_driven = tk.IntVar(value=0)
LED_driven = tk.IntVar(value=0)
High_speed_camera_driven = tk.IntVar(value=1)
checkbutton = tk.Checkbutton(root,text='Camera Driven',font='Helvetica 12',variable=Camera_driven).place(relx = .58,rely = .28)
checkbutton = tk.Checkbutton(root,text='LED Driven',font='Helvetica 12',variable=LED_driven).place(relx = .70,rely = .28)       
checkbutton = tk.Checkbutton(root,text='High Speed Camera',font='Helvetica 12',variable=High_speed_camera_driven).place(relx = .58,rely = .32)       

should_record = tk.IntVar()
checkbutton = tk.Checkbutton(root,text='Record',font='Helvetica 12',variable=should_record).grid(row = 0,column = 13)     

# ------ Initialize GUI (with safety check) ------ #

make_waveform(0) #ii=0
make_temperature_plot()
grab_image_for_gui(0) #ii=0

set_power_supply_voltage(0)
turn_off_AD_5V_supply()

#root.quit() #stops main loop and could be used to load in videos after; or put video place holders and .config update it later

# ------ Run GUI (with safety keyboard interrupt) ------ #

try:
    root.mainloop()
except KeyboardInterrupt: #allows safety even if stop the code with "control C" keyboard interrupt
    print("Interrupted")
    set_power_supply_voltage(0)
    turn_off_AD_5V_supply()
    close_basler_camera()
    close_AD2()


